#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#  collar.py
#  collar
#

import click

from collar import fleet, filters, errors

TEST_FLEET = None
TEST_MODE = False


@click.group()
@click.option('--region', default='us-east-1',
              help='The EC2 region to use.')
@click.pass_context
def main(ctx, region):
    """
    Query and access EC2 instances by their tags and other instance
    parameters. Use one of the subcommands below.
    """
    ctx.obj = {'region': region}


@main.command(name='list')
@click.argument('criteria', nargs=-1)
@click.option('--region', help='The EC2 region to use.')
@click.option('--public', is_flag=True,
              help='List public domain names instead of ids.')
@click.option('--private', is_flag=True,
              help='List private domain names instead of ids.')
@click.pass_context
def list_cmd(ctx, criteria, region=None, public=False, private=False):
    """
    List the instance id of every instance on your account that matches your
    search criteria. If no criteria are provided, all instances are returned.
    """
    if public and private:
        bail('cannot use --public and --private together', ctx)

    region = region or ctx.obj['region']

    try:
        list_checked(region, criteria, public=public, private=private)
    except errors.UserError as e:
        bail(e.message, ctx)


def list_checked(region, criteria, public=False, private=False):
    instances = TEST_FLEET or fleet.Fleet(region)
    is_included = filters.compile(criteria)
    matches = filter(is_included, instances)

    if public:
        show = lambda i: i['PublicDnsName']
    elif private:
        show = lambda i: i['PrivateDnsName']
    else:
        show = lambda i: i['Id']

    for i in matches:
        s = show(i)
        if s:
            click.echo(s)


@main.command(name='describe')
@click.argument('criteria', nargs=-1)
@click.option('--region', help='The EC2 region to use.')
@click.pass_context
def describe_cmd(ctx, criteria, region=None):
    """
    Describe all the servers which match your given criteria. If no criteria
    are given, all servers in the account will be listed.
    """
    region = region or ctx.obj['region']

    try:
        describe_checked(region, criteria)
    except errors.UserError as e:
        bail(e.message, ctx)


def describe_checked(region, criteria):
    instances = TEST_FLEET or fleet.Fleet(region)
    is_included = filters.compile(criteria)
    matches = filter(is_included, instances)

    for match in matches:
        click.echo(describe_instance(match))


def describe_instance(i):
    lines = []
    for k, v in sorted(i.items()):
        if not k.startswith('__') and v is not None:
            k_s = k + ':'
            if not TEST_MODE:
                k_s = click.style(k_s, fg='blue')

            v_s = (','.join(v)
                   if isinstance(v, list)
                   else v)
            lines.append('{} {}'.format(k_s, v_s))
    lines.append('')
    return '\n'.join(lines)


def bail(message, ctx):
    click.echo(message, err=True)
    ctx.exit(1)


if __name__ == '__main__':
    main()
